varying vec2 LightmapCoords;
varying vec2 texcoord;
varying vec4 glcolor;
attribute vec4 mc_Entity;
attribute vec2 mc_midTexCoord;
flat varying float material;
varying vec3 ViewPos;
varying vec3 MixedLights;

#include "/lib/settings.glsl"
#include "/lib/tonemap.glsl"

#include "/lib/util.glsl"
#include "/global/sky.glsl"
#include "/lib/blur.glsl"

// [0] is direct, [1] is ambient
vec3[2] get_sun_color() {
    vec3[2] FinalSun;

    #ifdef DIMENSION_NETHER
    FinalSun[1] = vec3(0.55, 0.4, 0.3);
    FinalSun[0] = vec3(0);
    return FinalSun;
    #elif defined DIMENSION_END
    FinalSun[1] = vec3(0.2, 0.2, 0.35);
    FinalSun[0] = vec3(0.175, 0.1, 0.2);
    return FinalSun;
    #endif
    const vec3 SUNRISE_SUN = vec3(SUNRISE_RED, SUNRISE_GREEN, SUNRISE_BLUE);
    const vec3 NOON_SUN = vec3(NOON_RED, NOON_GREEN, NOON_BLUE);
    const vec3 SUNSET_SUN = vec3(SUNSET_RED, SUNSET_GREEN, SUNSET_BLUE);

    FinalSun[0] = mix(mix(mix(SUNSET_SUN, vec3(0), nightStrength), NOON_SUN, dayStrength), SUNRISE_SUN, sunriseStrength);
    vec3[2] SkyCol = calc_sky_coef();
    FinalSun[1] = vec3(mix(mix(mix(SUNSET_AMBIENT, NIGHT_AMBIENT, nightStrength), NOON_AMBIENT, dayStrength), SUNRISE_AMBIENT, sunriseStrength));
    FinalSun[1] = mix(FinalSun[1], mix(SkyCol[0], SkyCol[1], 0.7), 0.4);

    FinalSun[0] = apply_saturation(FinalSun[0], 1 - rainStrength/2) * (1 - rainStrength*0.33);
    return FinalSun;
}

vec3 obscure_moonlight(vec3 LightColor, float NdotL) {

    LightColor = apply_saturation(LightColor, 1 - rainStrength/2) * (1 - rainStrength*0.33);
    return LightColor*NdotL*nightStrength;
}

uniform float framemod8;

vec2 jitterOffsets[8] = vec2[8](
							vec2( 0.125,-0.375),
							vec2(-0.125, 0.375),
							vec2( 0.625, 0.125),
							vec2( 0.375,-0.625),
							vec2(-0.625, 0.625),
							vec2(-0.875,-0.125),
							vec2( 0.375,-0.875),
							vec2( 0.875, 0.875)
						);

void init_generic() {
    gl_Position = ftransform();
    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    LightmapCoords = mat2(gl_TextureMatrix[1]) * gl_MultiTexCoord1.st;
    LightmapCoords = (LightmapCoords * 33.05f / 32.0f) - (1.05f / 32.0f);
    material = mc_Entity.x;
    ViewPos = (gl_ModelViewMatrix * gl_Vertex).xyz;
    vec3 Normal;
    if(material == 10001 || material == 10004 || material == 10005 || material == 10006) {
        Normal = normalize(upPosition);
        // Make grass darker at the bottom. It looks better this way
        if(gl_MultiTexCoord0.t > mc_midTexCoord.t && (material == 10004 || material == 10005)) Normal *= 0.5;
    }
    else {
        Normal = normalize(gl_NormalMatrix * gl_Normal);
    }
    glcolor = gl_Color;
    float Dist = length(ViewPos);
    
    float NdotLSun, NdotLMoon;
    #ifndef DIMENSION_NETHER
    NdotLSun = max(dot(Normal, normalize(sunPosition)), 0);
    NdotLMoon = max(dot(Normal, normalize(moonPosition)), 0);
    #endif

    #ifdef HANDHELD_LIGHTS
    float HandheldLight = max(heldBlockLightValue, heldBlockLightValue2)/15.0;
    LightmapCoords.x = max(LightmapCoords.x, HandheldLight-Dist/15.0);
    #endif

    vec3 TorchColor = vec3(LM_RED, LM_GREEN, LM_BLUE);
    vec3 MinLight = vec3(MIN_LIGHT_AMOUNT-0.01, MIN_LIGHT_AMOUNT-0.025, MIN_LIGHT_AMOUNT);
    MinLight += nightVision/3;
     
    #ifndef DIMENSION_OVERWORLD
        LightmapCoords.y = 1;
    #endif

    vec3[2] FinalSun = get_sun_color();
    const vec3 MoonlightColor = vec3(MOON_RED, MOON_GREEN, MOON_BLUE);

    #ifdef DIMENSION_OVERWORLD
        float TorchNerfFactor = min(get_luminance(FinalSun[1])*2, 1);

    #elif defined DIMENSION_END
        float TorchNerfFactor = 0.66;
    #else
        float TorchNerfFactor = 0;
    #endif

    #ifndef DIMENSION_NETHER
    // This cuts off direct sunlight it semi-occulded areas, but only if they are facing up.
    float FakeShadowFactor = smoothstep(0.75, 0.9, LightmapCoords.y);

    FinalSun[1] += (FinalSun[0] * NdotLSun + obscure_moonlight(MoonlightColor, NdotLMoon)) * FakeShadowFactor;
    #endif 

    LightmapCoords.x *= LightmapCoords.x;

    LightmapCoords.x = mix(LightmapCoords.x, LightmapCoords.x*(1-TorchNerfFactor), LightmapCoords.y);
    MixedLights = TorchColor * LightmapCoords.x + FinalSun[1] * LightmapCoords.y;
    MixedLights = mix(MinLight, MixedLights, max(LightmapCoords.x, LightmapCoords.y));
    MixedLights *= 1-darknessLightFactor;
}
