#include "/lib/noise.glsl"

// [0] is top, [1] is ground
vec3[2] calc_sky_coef() {
    vec3[2] FinalSky;
    const vec3 SKY_TOP_NOON = vec3(NOON_SKY_T_R, NOON_SKY_T_G, NOON_SKY_T_B);
    const vec3 SKY_TOP_SUNRISE = vec3(SUNRISE_SKY_T_R, SUNRISE_SKY_T_G, SUNRISE_SKY_T_B);
    const vec3 SKY_TOP_SUNSET = vec3(SUNSET_SKY_T_R, SUNSET_SKY_T_G, SUNSET_SKY_T_B);
    const vec3 SKY_TOP_NIGHT = vec3(NIGHT_SKY_T_R, NIGHT_SKY_T_G, NIGHT_SKY_T_B);

    const vec3 SKY_GROUND_NOON = vec3(NOON_SKY_G_R, NOON_SKY_G_G, NOON_SKY_G_B);
    const vec3 SKY_GROUND_SUNRISE = vec3(SUNRISE_SKY_G_R, SUNRISE_SKY_G_G, SUNRISE_SKY_G_B);
    const vec3 SKY_GROUND_SUNSET = vec3(SUNSET_SKY_G_R, SUNSET_SKY_G_G, SUNSET_SKY_G_B);
    const vec3 SKY_GROUND_NIGHT = vec3(NIGHT_SKY_G_R, NIGHT_SKY_G_G, NIGHT_SKY_G_B);

    FinalSky[0] = mix(mix(mix(SKY_TOP_SUNSET, SKY_TOP_NIGHT, nightStrength), SKY_TOP_NOON, dayStrength), SKY_TOP_SUNRISE, sunriseStrength);
    FinalSky[1] = mix(mix(mix(SKY_GROUND_SUNSET, SKY_GROUND_NIGHT, nightStrength), SKY_GROUND_NOON, dayStrength), SKY_GROUND_SUNRISE, sunriseStrength);

    FinalSky[0] = apply_saturation(FinalSky[0], 1 - rainStrength/2);
    FinalSky[1] = apply_saturation(FinalSky[1], 1 - rainStrength/2)* (1 - rainStrength*0.33);
    return FinalSky;
} 

float fogify(float x, float w) {
	return w / (x * x + w);
}

vec2 rotate(vec2 P, float Ang) {
    float cosT = cos(Ang);
    float sinT = sin(Ang);
    return vec2(
        P.x*cosT-P.y*sinT,
        P.y*cosT+P.x*sinT
    );
}

vec3 get_clouds(vec3 CloudPos, vec3 SkyColor) {
    const int ACTUAL_CLOUD_SPEED = 75-CLOUD_SPEED*10;
    float Animation = float(worldTime)/20/ACTUAL_CLOUD_SPEED;

    float Noise = fbm_clouds(CloudPos.xz*CLOUD_SIZE+Animation, CLOUD_QUALITY);
    Noise = pow(Noise, CLOUD_AMOUNT - rainStrength*6)*1500;

    Noise = 1-1/(1+Noise); // This looks good for some reason
    vec3 CloudColor = calc_sky_coef()[1];
    CloudColor *= 3-exp(Noise * 0.8);
    Noise *= smoothstep(0.0, 0.2, CloudPos.y); // Reduce cloud amount further away

	return mix(SkyColor, CloudColor, Noise);
}

vec3 get_sky(vec3 pos, vec3 SunVec) {
	float upDot = dot(pos, gbufferModelView[1].xyz)+0.1; //not much, what's up with you?
    vec3[2] FinalSky = calc_sky_coef();

    // Sun glare
    float Dist = clamp(dot(pos, SunVec), 0, 1);
    Dist *= Dist;
    float Visibility = cos(timeAngle*2*PI); 
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility *= Visibility;
    Visibility *= Visibility;
    
    const vec3 SUN_GLARE = vec3(0.7, 0.25, 0.0);
    FinalSky[1] += SUN_GLARE*Dist*Visibility;

	return mix(FinalSky[0], FinalSky[1], fogify(max(upDot, 0.0), 0.05));
}

float get_stars(vec3 PlayerPos) {
    vec3 StarCoord = PlayerPos / (PlayerPos.y + length(PlayerPos.xz));
    StarCoord.x += frameTimeCounter*0.001;
	const float ACTUAL_STAR_SIZE = STAR_SIZE * 512;
	StarCoord = floor(StarCoord * ACTUAL_STAR_SIZE) / ACTUAL_STAR_SIZE;

    float Visibility = clamp(sqrt(StarCoord.y+0.1), 0, 1);
    #ifdef DIMENSION_OVERWORLD
    Visibility *= nightStrength;
    #endif
	return pow(random(StarCoord.xz), 1024) * Visibility * STAR_STRENGTH;
}

vec3 get_end_sky(vec3 pos, vec3 PlayerPos) {
    const vec3 SkyT = vec3(END_SKY_T_R, END_SKY_T_G, END_SKY_T_B);
    const vec3 SkyG1 = vec3(END_SKY_G_R, END_SKY_G_G, END_SKY_G_B);
    const vec3 SkyG2 = vec3(END_SKY_G_RA, END_SKY_G_GA, END_SKY_G_BA);
	float upDot = dot(pos, gbufferModelView[1].xyz); //not much, what's up with you?
	vec2 RotPos1 = rotate(PlayerPos.xz, frameTimeCounter*0.02);
	vec2 RotPos2 = rotate(PlayerPos.xz, -frameTimeCounter*0.007);
	float Noise1 = fbm_fast(RotPos1*16, 2)*(1-abs(upDot));
	float Noise2 = fbm_fast(RotPos2*28, 2)*(1-abs(upDot));
    float Noise = (Noise1+Noise2);
    vec3 SkyG = SkyG1*Noise1+SkyG2*Noise2;
    vec3 Final = SkyT+SkyG*(fogify(upDot+0.2, 0.05));
    Final *= max(1-fogify(max(upDot+0.3, 0), 0.02), 0.01);
	return Final;
}

