#include "/lib/all_the_uniforms.glsl"
varying vec2 texcoord;
#include "/lib/util.glsl"
#include "/lib/tonemap.glsl"
#include "/lib/blur.glsl"
#include "/lib/settings.glsl"
#include "/lib/taa.glsl"

// Color grading, Bloom final, TAA final

vec3 film_grain(vec3 Color, vec2 Pos) {
    Pos.x += fract(float(frameCounter)/4.14159)*234;
    Pos.y -= fract(float(frameCounter)/5.49382)*567;
    Color += (texelFetch(noisetex, ivec2(Pos.xy) & 255, 0).xyz - 0.5)*FILM_GRAIN_STRENGTH;
    return Color;
}

vec3 apply_vignette(vec3 Color, vec2 Pos) {
	Pos = Pos - 0.5;
	float Strength = length(Pos*VIGNETTE_OPACITY);
	Strength = pow(Strength, 4-VIGNETTE_FALLOFF);
	Color *= 1-min(Strength, 1);
	return Color;
}

/* DRAWBUFFERS:03 */

void main() {
	vec4 Color = texture2D(colortex0, texcoord);
	#if TAA_MODE != 0
	gl_FragData[1].xyz = TAA(Color.rgb);
	#endif
	#ifdef BLOOM 
		float Offset = 0;
		vec3 FinalBloom = vec3(0);
		for(int i = 2; i <= 2+BLOOM_SIZE; i++) {
			FinalBloom += texture2D(colortex1, texcoord/pow(2, i)+Offset).xyz/i;
			Offset += 1/pow(2, i);
		}
		Color.rgb = mix(Color.rgb, FinalBloom, clamp(get_luminance(FinalBloom)-BLOOM_THRESHOLD, 0, 1)*BLOOM_STRENGTH);
	#endif

	#ifdef FILM_GRAIN
	Color.rgb = film_grain(Color.rgb, gl_FragCoord.xy);
	#endif

	Color.rgb = ACESFilm(Color.rgb);

	Color.rgb = apply_vibrance(Color.rgb, VIBRANCE);
	Color.rgb = apply_saturation(Color.rgb, SATURATION);
	Color.rgb = apply_contrast(Color.rgb, CONTRAST);

	vec3 MinBright = vec3(TONEMAP_MIN_R, TONEMAP_MIN_G, TONEMAP_MIN_B);
	Color.rgb = max(Color.rgb, MinBright);

	#if VIGNETTE_OPACITY > 0.0
	Color.rgb = apply_vignette(Color.rgb, texcoord);
	#endif

	gl_FragData[0] = Color;
	#ifdef DEBUG_BUFFER
	gl_FragData[0] = texture2D(colortex1, texcoord);
	#endif
}
